local _name, _addon = ...;
local cases = _addon.cases;

local debug = nil;

local DEFAULT_CASE = "Common";
local DO_ASK = 0;
local DO_ACCEPT = 1;
local DO_DENY = 2;
local DO_ALT = 3;
local DO_HIDE = 9;
local DO_NOTHING = "error, blocked";

local BACKDROP = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	edgeSize = 12,
	insets = { top = 2, right = 2, left = 2, bottom = 2, },
};
local color_fg = { 1, 0.5, 0, 1.0 };
local color_bg = { 0, 0, 0, 1};
local gap = 10;

local function am(msg)
	msg = msg or "nil";
	DEFAULT_CHAT_FRAME:AddMessage(msg);
end

----------------------------------------------------------------------------------------
local TIME_DELAY = 0.1;

local Timer = CreateFrame("Frame", "Timer", UIParent);
Timer:UnregisterAllEvents();

function Timer:DelayExecute(dialog, func)
	if not func then return; end
	
	self.elapsed = 0;
	self:SetScript("OnUpdate", function(self, elapsed)
		self.elapsed = self.elapsed + elapsed;
		if self.elapsed < TIME_DELAY then return; end
		self:SetScript("OnUpdate", nil);
		
		local success, rtn = pcall(func, dialog);
		if not success then
			if debug then
				am("Insecure attempt failed on : "..dialog.which);
				am(tostring(rtn));
			end
			LousyConfirmDB[dialog.which] = DO_NOTHING;
		end
	end);
end

function _addon:DelayExecute(dialog, func)
	Timer:DelayExecute(dialog, func);
end

----------------------------------------------------------------------------------------
local addon = CreateFrame("Frame", "LoustConfirm", UIParent);
addon.btns = {};
addon:UnregisterAllEvents();
addon:Hide();

function addon:ADDON_LOADED(name)
	if name ~= _name then
		return;
	end
	self:UnregisterEvent("ADDON_LOADED");

	if not LousyConfirmDB then
		LousyConfirmDB = {};
	end
	self:RegisterEvent("PLAYER_LOGIN");
end

function addon:PLAYER_LOGIN()
	self:UnregisterEvent("PLAYER_LOGIN");

	self:SetScript("OnShow", self.OnShow);
	self:init();
	self:doHook();
end

function addon:doHook()
	hooksecurefunc("StaticPopup_Show", function(which, ...)
		if not which then return; end
		local dialog = StaticPopup_FindVisible(which);
		if not dialog then return; end

		self.which = which;
		local actionID = LousyConfirmDB[which];
		if LousyConfirmDB.disabled or IsAltKeyDown() or not actionID then
			--PlaySound("igMainMenuOpen");
			self:showOptions(dialog, actionID);
		elseif actionID == DO_NOTHING then
			--PlaySound("igMainMenuOpen");
		else
			local func = cases[which] and cases[which][actionID];
			func = func or cases[DEFAULT_CASE][actionID];
			local success, rtn = pcall(func, dialog, ...);
			if not success then
				if debug then
					am("Secure attempt failed on : "..which);
					am(tostring(rtn));
				end
				Timer:DelayExecute(dialog, func);
			end
		end
	end);
	hooksecurefunc("StaticPopup_OnHide", function(dialog)
		if not dialog.which or dialog.which ~= self.which then return; end
		self.which = nil;
		self:Hide();
	end);
	--[[PlaySound hijack
	local _PlaySound = PlaySound;
	PlaySound = function(sound, soundChannel)
		if sound == "igMainMenuOpen" and not self.which then
			return;
		elseif sound == "igMainMenuClose" and self.which then
			return;
		end
		_PlaySound(sound, soundChannel);
	end]]
end

function addon:checked(button)
	local state = button:GetChecked();
	for _, btn in ipairs(self.btns) do
		btn:SetChecked(btn == button and state);
	end
	if self.which then
		LousyConfirmDB[self.which] = state and button.value;
	end
end

function addon:showOptions(dialog, value)
	for _, btn in ipairs(self.btns) do
		btn:ClearAllPoints();
		btn:Hide();
	end
	wipe(self.btns);

	local which = dialog.which;
	
	self.title:SetText(which);
	local ptr = self.title;
	
	local case = cases[which];
	if case and case.info then
		self.text:SetText(case.info);
		self.text:Show();
		ptr = self.text;
	else
		self.text:Hide();
	end
	
	if not (case and case.blocked) then
		local info = StaticPopupDialogs[which];
		if info then
			if info.button1 and not (info.DisplayButton1 and not info.DisplayButton1(dialog)) then
				self.check_accept:SetPoint("TOPLEFT", ptr, "BOTTOMLEFT", 0, -gap / 2);
				self.check_accept.text:SetText(info.button1);
				self.check_accept:Show();
				tinsert(self.btns, self.check_accept);
				ptr = self.check_accept;
			end
			if info.button3 and not (info.DisplayButton3 and not info.DisplayButton3(dialog)) then
				self.check_alt:SetPoint("TOPLEFT", ptr, "BOTTOMLEFT", 0, -gap / 2);
				self.check_alt.text:SetText(info.button3);
				self.check_alt:Show();
				tinsert(self.btns, self.check_alt);
				ptr = self.check_alt;
			end
			if info.button2 and not (info.DisplayButton2 and not info.DisplayButton2(dialog)) then
				self.check_deny:SetPoint("TOPLEFT", ptr, "BOTTOMLEFT", 0, -gap / 2);
				self.check_deny.text:SetText(info.button2);
				self.check_deny:Show();
				tinsert(self.btns, self.check_deny);
				ptr = self.check_deny;
			end
		end
		if #self.btns == 0 then
			self.check_hide:SetPoint("TOPLEFT", ptr, "BOTTOMLEFT", 0, -gap / 2);
			self.check_hide:Show();
			tinsert(self.btns, self.check_hide);
			ptr = self.check_hide;
		end
		
		self.check_ignore:SetPoint("TOPLEFT", ptr, "BOTTOMLEFT", 0, -gap / 2);
		self.check_ignore:Show();
		tinsert(self.btns, self.check_ignore);
		ptr = self.check_ignore;
	end

	self:SetParent(dialog);
	self:SetPoint("TOPLEFT", dialog, "TOPRIGHT");
	self:Show();
end

function addon:OnShow()
	local width = math.max(self.title:GetStringWidth(), 160);
	for _, btn in ipairs(self.btns) do
		btn:SetChecked(btn.value == value);
		width = math.max(width, btn:GetWidth() + btn.text:GetStringWidth() + gap);
	end
	self:SetWidth(width + 2 * gap);
	
	local bottom = self.title:GetBottom();
	if self.text:IsShown() then
		local _, size = self.text:GetFont();
		self.text:SetHeight(math.ceil(self.text:GetStringWidth() / width) * size);
		bottom = self.text:GetBottom();
	end
	for _, btn in ipairs(self.btns) do
		bottom = math.min(bottom, btn:GetBottom());
	end
	self:SetHeight(self:GetTop() - bottom + gap);
end

function addon:init()
	self:SetClampedToScreen(false);
	self:SetBackdrop(BACKDROP);
	self:SetBackdropColor(unpack(color_bg));

	self:SetWidth(160);
	self:SetHeight(120);

	local title = self:CreateFontString(nil, "OVERLAY", "QuestFont");
	title:SetPoint("TOPLEFT", gap, -gap);
	title:SetPoint("TOPRIGHT", -gap , -gap);
	title:SetTextColor(unpack(color_fg));
	title:SetJustifyH("CENTER");
	title:SetJustifyV("CENTER");
	title:SetWordWrap(false);
	title:SetText("Title");
	self.title = title;

	local text = self:CreateFontString(nil, "OVERLAY", "SystemFont_Small");
	text:SetPoint("TOP", title, "BOTTOM");
	text:SetPoint("LEFT", gap, 0);
	text:SetPoint("RIGHT", -gap , 0);
	text:SetTextColor(1, 1, 1, 1);
	text:SetJustifyH("LEFT");
	text:SetWordWrap(true);
	self.text = text;
	
	local check_accept = CreateFrame("CheckButton", "$ParentAcceptCheck", self, "OptionsSmallCheckButtonTemplate");
	check_accept.value = DO_ACCEPT;
	check_accept:SetScript("OnClick", function(self)
		addon:checked(self);
	end);
	local text_accept = _G[check_accept:GetName().."Text"];
	text_accept:SetText("Always Accept");
	text_accept:SetWordWrap(false);
	check_accept.text = text_accept;
	self.check_accept = check_accept;

	local check_deny = CreateFrame("CheckButton", "$ParentDenyCheck", self, "OptionsSmallCheckButtonTemplate");
	check_deny.value = DO_DENY;
	check_deny:SetScript("OnClick", function(self)
		addon:checked(self);
	end);
	local text_deny = _G[check_deny:GetName().."Text"];
	text_deny:SetText("Always Deny");
	text_deny:SetWordWrap(false);
	check_deny.text = text_deny;
	self.check_deny = check_deny;

	local check_alt = CreateFrame("CheckButton", "$ParentAltCheck", self, "OptionsSmallCheckButtonTemplate");
	check_alt.value = DO_ALT;
	check_alt:SetScript("OnClick", function(self)
		addon:checked(self);
	end);
	local text_alt = _G[check_alt:GetName().."Text"];
	text_alt:SetText("Always Alternative");
	text_alt:SetWordWrap(false);
	check_alt.text = text_alt;
	self.text_alt = text_alt;

	local check_ignore = CreateFrame("CheckButton", "$ParentIgnoreCheck", self, "OptionsSmallCheckButtonTemplate");
	check_ignore.value = DO_ASK;
	check_ignore:SetScript("OnClick", function(self)
		addon:checked(self);
	end);
	local text_ignore = _G[check_ignore:GetName().."Text"];
	text_ignore:SetText("Leave it as is");
	text_ignore:SetWordWrap(false);
	check_ignore.text = text_ignore;
	self.check_ignore = check_ignore;

	local check_hide = CreateFrame("CheckButton", "$ParentHideCheck", self, "OptionsSmallCheckButtonTemplate");
	check_hide.value = DO_HIDE;
	check_hide:SetScript("OnClick", function(self)
		addon:checked(self);
	end);
	local text_hide = _G[check_hide:GetName().."Text"];
	text_hide:SetText("Always Hide");
	text_hide:SetWordWrap(false);
	check_hide.text = text_hide;
	self.check_hide = check_hide;
end

----------------------------------------------------------------------------------------
addon:SetScript("OnEvent", function(self, event, ...)
	local f = self[event];
	if f then
		f(self,...);
	else
		utils:am(tostring(event));
	end
end);
----------------------------------------------------------------------------------------
addon:RegisterEvent("ADDON_LOADED");

StaticPopupDialogs["SORT_OF_CONFIRM_ID"] = {
	text = "Confirm this confirm that, seriously.",
	button1 = "Why not!",
	button2 = "Nah",
	OnAccept = function (self) end,
	OnCancel = function (self) end,
	OnHide = function (self) end,
	hideOnEscape = 1,
	timeout = 0,
	exclusive = 1,
	whileDead = 1,
};

SLASH_LOUSYCONFIRM1 = "/lc";
SlashCmdList.LOUSYCONFIRM = function(msg, editBox)
	if msg == "reset" then
		LousyConfirmDB = {};
	elseif msg == "debug" then
		debug = not debug;
		am("LousyConfirm: debug mode ".. (debug and "on" or "off"));
	elseif msg == "test" then
		StaticPopup_Show("SORT_OF_CONFIRM_ID");
	else
		LousyConfirmDB.disabled = not LousyConfirmDB.disabled;
		am("LousyConfirm: ".. (LousyConfirmDB.disabled and "disabled" or "enabled"));
	end
end